@@ -2,7 +2,7 @@ from datetime import timedelta
 from typing import List, Union
 
 from flask_login import current_user
-from isodate import datetime_isoformat, duration_isoformat
+from isodate import datetime_isoformat
 from marshmallow import fields, post_load, validates_schema, ValidationError
 from marshmallow.validate import OneOf
 from marshmallow_polyfield import PolyField
@@ -16,7 +16,7 @@ from flexmeasures.api.common.schemas.sensors import SensorField
 from flexmeasures.api.common.utils.api_utils import upsample_values
 from flexmeasures.data.schemas.times import AwareDateTimeField, DurationField
 from flexmeasures.data.services.time_series import simplify_index
-from flexmeasures.utils.time_utils import server_now
+from flexmeasures.utils.time_utils import duration_isoformat, server_now
 from flexmeasures.utils.unit_utils import (
     convert_units,
     units_are_convertible,
@@ -97,6 +97,8 @@ class SensorDataDescriptionSchema(ma.Schema):
 
 class GetSensorDataSchema(SensorDataDescriptionSchema):
 
+    resolution = DurationField(required=False)
+
     # Optional field that can be used for extra validation
     type = fields.Str(
         required=False,
@@ -118,14 +120,10 @@ class GetSensorDataSchema(SensorDataDescriptionSchema):
     def check_schema_unit_against_type(self, data, **kwargs):
         requested_unit = data["unit"]
         _type = data.get("type", None)
-        if (
-            _type
-            in (
-                "GetMeterDataRequest",
-                "GetPrognosisRequest",
-            )
-            and not units_are_convertible(requested_unit, "MW")
-        ):
+        if _type in (
+            "GetMeterDataRequest",
+            "GetPrognosisRequest",
+        ) and not units_are_convertible(requested_unit, "MW"):
             raise ValidationError(
                 f"The unit requested for this message type should be convertible from MW, got incompatible unit: {requested_unit}"
             )
@@ -145,12 +143,14 @@ class GetSensorDataSchema(SensorDataDescriptionSchema):
         - converts to a single deterministic belief per event
         - ensures the response respects the requested time frame
         - converts values to the requested unit
+        - converts values to the requested resolution
         """
         sensor: Sensor = sensor_data_description["sensor"]
         start = sensor_data_description["start"]
         duration = sensor_data_description["duration"]
         end = sensor_data_description["start"] + duration
         unit = sensor_data_description["unit"]
+        resolution = sensor_data_description.get("resolution")
 
         # Post-load configuration of belief timing against message type
         horizons_at_least = sensor_data_description.get("horizon", None)
@@ -173,13 +173,14 @@ class GetSensorDataSchema(SensorDataDescriptionSchema):
                 horizons_at_most=horizons_at_most,
                 beliefs_before=sensor_data_description.get("prior", None),
                 one_deterministic_belief_per_event=True,
+                resolution=resolution,
                 as_json=False,
             )
         )
 
         # Convert to desired time range
         index = pd.date_range(
-            start=start, end=end, freq=sensor.event_resolution, closed="left"
+            start=start, end=end, freq=df.event_resolution, closed="left"
         )
         df = df.reindex(index)
 
@@ -240,14 +241,10 @@ class PostSensorDataSchema(SensorDataDescriptionSchema):
     def check_schema_unit_against_type(self, data, **kwargs):
         posted_unit = data["unit"]
         _type = data.get("type", None)
-        if (
-            _type
-            in (
-                "PostMeterDataRequest",
-                "PostPrognosisRequest",
-            )
-            and not units_are_convertible(posted_unit, "MW")
-        ):
+        if _type in (
+            "PostMeterDataRequest",
+            "PostPrognosisRequest",
+        ) and not units_are_convertible(posted_unit, "MW"):
             raise ValidationError(
                 f"The unit required for this message type should be convertible to MW, got incompatible unit: {posted_unit}"
             )
