@@ -11,8 +11,10 @@ from flexmeasures.utils.entity_address_utils import (
     parse_entity_address,
     EntityAddressException,
 )
-from flexmeasures.data.models.assets import Asset, Power
-from flexmeasures.data.services.resources import get_assets
+from flexmeasures.data import db
+from flexmeasures.data.models.time_series import Sensor, TimedBelief
+from flexmeasures.data.queries.data_sources import get_or_create_source
+from flexmeasures.data.services.resources import get_sensors
 from flexmeasures.data.services.forecasting import create_forecasting_jobs
 from flexmeasures.api.common.responses import (
     invalid_domain,
@@ -24,8 +26,7 @@ from flexmeasures.api.common.responses import (
 )
 from flexmeasures.api.common.utils.api_utils import (
     groups_to_dict,
-    get_or_create_user_data_source,
-    save_to_db,
+    save_and_enqueue,
 )
 from flexmeasures.api.common.utils.validators import (
     type_accepted,
@@ -50,7 +51,7 @@ from flexmeasures.api.common.utils.validators import (
 )
 @optional_prior_accepted(ex_post=True, infer_missing=False)
 @period_required
-@get_data_downsampling_allowed("connection")
+@get_data_downsampling_allowed("connection", "fm0")
 @as_json
 def get_meter_data_response(
     unit,
@@ -97,9 +98,11 @@ def get_meter_data_response(
 @units_accepted("power", "MW")
 @assets_required("connection")
 @values_required
-@optional_horizon_accepted(ex_post=True, accept_repeating_interval=True)
+@optional_horizon_accepted(
+    ex_post=True, infer_missing=True, accept_repeating_interval=True
+)
 @period_required
-@post_data_checked_for_required_resolution("connection")
+@post_data_checked_for_required_resolution("connection", "fm0")
 @as_json
 def post_meter_data_response(
     unit,
@@ -160,13 +163,11 @@ def collect_connection_and_value_groups(
     Returns value sign in accordance with USEF specs
     (with negative production and positive consumption).
     """
-    from flask import current_app
-
     current_app.logger.info("GETTING")
-    user_assets = get_assets()
-    if not user_assets:
+    user_sensors = get_sensors()
+    if not user_sensors:
         current_app.logger.info("User doesn't seem to have any assets")
-    user_asset_ids = [asset.id for asset in user_assets]
+    user_sensor_ids = [sensor.id for sensor in user_sensors]
 
     end = start + duration
     value_groups = []
@@ -175,37 +176,42 @@ def collect_connection_and_value_groups(
     )  # Each connection in the old connection groups will be interpreted as a separate group
     for connections in connection_groups:
 
-        # Get the asset names
-        asset_names: List[str] = []
+        # Get the sensor names
+        sensor_names: List[str] = []
         for connection in connections:
 
             # Parse the entity address
             try:
                 connection_details = parse_entity_address(
-                    connection, entity_type="connection"
+                    connection, entity_type="connection", fm_scheme="fm0"
                 )
             except EntityAddressException as eae:
                 return invalid_domain(str(eae))
-            asset_id = connection_details["asset_id"]
+            sensor_id = connection_details["asset_id"]
 
-            # Look for the Asset object
-            if asset_id in user_asset_ids:
-                asset = Asset.query.filter(Asset.id == asset_id).one_or_none()
+            # Look for the Sensor object
+            if sensor_id in user_sensor_ids:
+                sensor = Sensor.query.filter(Sensor.id == sensor_id).one_or_none()
             else:
                 current_app.logger.warning("Cannot identify connection %s" % connection)
                 return unrecognized_connection_group()
-            asset_names.append(asset.name)
+            sensor_names.append(sensor.name)
 
         # Get the power values
         # TODO: fill NaN for non-existing values
-        power_bdf_dict: Dict[str, tb.BeliefsDataFrame] = Power.collect(
-            generic_asset_names=asset_names,
-            query_window=(start, end),
+        power_bdf_dict: Dict[str, tb.BeliefsDataFrame] = TimedBelief.search(
+            sensor_names,
+            event_starts_after=start,
+            event_ends_before=end,
             resolution=resolution,
-            belief_horizon_window=belief_horizon_window,
-            belief_time_window=belief_time_window,
+            horizons_at_least=belief_horizon_window[0],
+            horizons_at_most=belief_horizon_window[1],
+            beliefs_after=belief_time_window[0],
+            beliefs_before=belief_time_window[1],
             user_source_ids=user_source_ids,
             source_types=source_types,
+            most_recent_beliefs_only=True,
+            one_deterministic_belief_per_event=True,
             sum_multiple=False,
         )
         # Todo: parse time window of power_bdf_dict, which will be different for requests that are not of the form:
@@ -242,12 +248,12 @@ def create_connection_and_value_groups(  # noqa: C901
 
     current_app.logger.info("POSTING POWER DATA")
 
-    data_source = get_or_create_user_data_source(current_user)
-    user_assets = get_assets()
-    if not user_assets:
+    data_source = get_or_create_source(current_user)
+    user_sensors = get_sensors()
+    if not user_sensors:
         current_app.logger.info("User doesn't seem to have any assets")
-    user_asset_ids = [asset.id for asset in user_assets]
-    power_measurements = []
+    user_sensor_ids = [sensor.id for sensor in user_sensors]
+    power_df_per_connection = []
     forecasting_jobs = []
     for connection_group, value_group in zip(generic_asset_name_groups, value_groups):
         for connection in connection_group:
@@ -255,33 +261,40 @@ def create_connection_and_value_groups(  # noqa: C901
             # TODO: get asset through util function after refactoring
             # Parse the entity address
             try:
-                connection = parse_entity_address(connection, entity_type="connection")
+                connection = parse_entity_address(
+                    connection, entity_type="connection", fm_scheme="fm0"
+                )
             except EntityAddressException as eae:
                 return invalid_domain(str(eae))
-            asset_id = connection["asset_id"]
+            sensor_id = connection["asset_id"]
 
-            # Look for the Asset object
-            if asset_id in user_asset_ids:
-                asset = Asset.query.filter(Asset.id == asset_id).one_or_none()
+            # Look for the Sensor object
+            if sensor_id in user_sensor_ids:
+                sensor = Sensor.query.filter(Sensor.id == sensor_id).one_or_none()
             else:
                 current_app.logger.warning("Cannot identify connection %s" % connection)
                 return unrecognized_connection_group()
 
             # Validate the sign of the values (following USEF specs with positive consumption and negative production)
-            if asset.is_pure_consumer and any(v < 0 for v in value_group):
+            if sensor.get_attribute("is_strictly_non_positive") and any(
+                v < 0 for v in value_group
+            ):
                 extra_info = (
                     "Connection %s is registered as a pure consumer and can only receive non-negative values."
-                    % asset.entity_address
+                    % sensor.entity_address
                 )
                 return power_value_too_small(extra_info)
-            elif asset.is_pure_producer and any(v > 0 for v in value_group):
+            elif sensor.get_attribute("is_strictly_non_negative") and any(
+                v > 0 for v in value_group
+            ):
                 extra_info = (
                     "Connection %s is registered as a pure producer and can only receive non-positive values."
-                    % asset.entity_address
+                    % sensor.entity_address
                 )
                 return power_value_too_big(extra_info)
 
-            # Create new Power objects
+            # Create a new BeliefsDataFrame
+            beliefs = []
             for j, value in enumerate(value_group):
                 dt = start + j * duration / len(value_group)
                 if rolling:
@@ -290,29 +303,31 @@ def create_connection_and_value_groups(  # noqa: C901
                     h = horizon - (
                         (start + duration) - (dt + duration / len(value_group))
                     )
-                p = Power(
-                    datetime=dt,
-                    value=value
+                p = TimedBelief(
+                    event_start=dt,
+                    event_value=value
                     * -1,  # Reverse sign for FlexMeasures specs with positive production and negative consumption
-                    horizon=h,
-                    asset_id=asset.id,
-                    data_source_id=data_source.id,
+                    belief_horizon=h,
+                    sensor=sensor,
+                    source=data_source,
                 )
-                power_measurements.append(p)
+
+                assert p not in db.session
+                beliefs.append(p)
+            power_df_per_connection.append(tb.BeliefsDataFrame(beliefs))
 
             # make forecasts, but only if the sent-in values are not forecasts themselves
             if horizon <= timedelta(
                 hours=0
-            ):  # Todo: replace 0 hours with whatever the moment of switching from ex-ante to ex-post is for this generic asset
+            ):  # Todo: replace 0 hours with whatever the moment of switching from ex-ante to ex-post is for this sensor
                 forecasting_jobs.extend(
                     create_forecasting_jobs(
-                        "Power",
-                        asset_id,
+                        sensor_id,
                         start,
                         start + duration,
                         resolution=duration / len(value_group),
-                        enqueue=False,
+                        enqueue=False,  # will enqueue later, after saving data
                     )
                 )
 
-    return save_to_db(power_measurements, forecasting_jobs)
+    return save_and_enqueue(power_df_per_connection, forecasting_jobs)
