@@ -3,6 +3,7 @@ from flask import Flask
 from flexmeasures.api.v3_0.sensors import SensorAPI
 from flexmeasures.api.v3_0.users import UserAPI
 from flexmeasures.api.v3_0.assets import AssetAPI
+from flexmeasures.api.v3_0.health import HealthAPI
 
 
 def register_at(app: Flask):
@@ -13,3 +14,4 @@ def register_at(app: Flask):
     SensorAPI.register(app, route_prefix=v3_0_api_prefix)
     UserAPI.register(app, route_prefix=v3_0_api_prefix)
     AssetAPI.register(app, route_prefix=v3_0_api_prefix)
+    HealthAPI.register(app, route_prefix=v3_0_api_prefix)
