@@ -15,6 +15,7 @@ import getpass
 from sqlalchemy.exc import IntegrityError
 from timely_beliefs.sensors.func_store.knowledge_horizons import x_days_ago_at_y_oclock
 import timely_beliefs as tb
+import timely_beliefs.utils as tb_utils
 from workalendar.registry import registry as workalendar_registry
 
 from flexmeasures.data import db
@@ -24,7 +25,7 @@ from flexmeasures.data.scripts.data_gen import (
     add_default_asset_types,
 )
 from flexmeasures.data.services.forecasting import create_forecasting_jobs
-from flexmeasures.data.services.scheduling import make_schedule
+from flexmeasures.data.services.scheduling import make_schedule, create_scheduling_job
 from flexmeasures.data.services.users import create_user
 from flexmeasures.data.models.user import Account, AccountRole, RolesAccounts
 from flexmeasures.data.models.time_series import (
@@ -703,9 +704,8 @@ def add_holidays(
 @click.option(
     "--as-job",
     is_flag=True,
-    help="Whether to queue a forecasting job instead of computing directly."
-    " Useful to run locally and create forecasts on a remote server. In that case, just point the redis db in your"
-    " config settings to that of the remote server. To process the job, run a worker to process the forecasting queue. Defaults to False.",
+    help="Whether to queue a forecasting job instead of computing directly. "
+    "To process the job, run a worker (on any computer, but configured to the same databases) to process the 'forecasting' queue. Defaults to False.",
 )
 def create_forecasts(
     sensor_ids: List[int],
@@ -720,7 +720,7 @@ def create_forecasts(
 
     For example:
 
-        --from_date 2015-02-02 --to_date 2015-02-04 --horizon_hours 6 --sensor-id 12 --sensor-id 14
+        --from-date 2015-02-02 --to-date 2015-02-04 --horizon 6 --sensor-id 12 --sensor-id 14
 
         This will create forecast values from 0am on May 2nd to 0am on May 5th,
         based on a 6-hour horizon, for sensors 12 and 14.
@@ -743,20 +743,23 @@ def create_forecasts(
         event_resolution = None
 
     if as_job:
+        num_jobs = 0
         for sensor_id in sensor_ids:
             for horizon in horizons:
                 # Note that this time period refers to the period of events we are forecasting, while in create_forecasting_jobs
                 # the time period refers to the period of belief_times, therefore we are subtracting the horizon.
-                create_forecasting_jobs(
+                jobs = create_forecasting_jobs(
                     sensor_id=sensor_id,
                     horizons=[horizon],
                     start_of_roll=forecast_start - horizon,
                     end_of_roll=forecast_end - horizon,
                 )
+                num_jobs += len(jobs)
+        print(f"{num_jobs} new forecasting job(s) added to the queue.")
     else:
         from flexmeasures.data.scripts.data_gen import populate_time_series_forecasts
 
-        populate_time_series_forecasts(
+        populate_time_series_forecasts(  # this function reports its own output
             db=app.db,
             sensor_ids=sensor_ids,
             horizons=horizons,
@@ -775,12 +778,26 @@ def create_forecasts(
     required=True,
     help="Create schedule for this sensor. Follow up with the sensor's ID.",
 )
+@click.option(
+    "--consumption-price-sensor",
+    "consumption_price_sensor",
+    type=SensorIdField(),
+    required=False,
+    help="Optimize consumption against this sensor. The sensor typically records an electricity price (e.g. in EUR/kWh), but this field can also be used to optimize against some emission intensity factor (e.g. in kg CO₂ eq./kWh). Follow up with the sensor's ID.",
+)
+@click.option(
+    "--production-price-sensor",
+    "production_price_sensor",
+    type=SensorIdField(),
+    required=False,
+    help="Optimize production against this sensor. Defaults to the consumption price sensor. The sensor typically records an electricity price (e.g. in EUR/kWh), but this field can also be used to optimize against some emission intensity factor (e.g. in kg CO₂ eq./kWh). Follow up with the sensor's ID.",
+)
 @click.option(
     "--optimization-context-id",
     "optimization_context_sensor",
     type=SensorIdField(),
-    required=True,
-    help="Optimize against this sensor, which measures a price factor or CO₂ intensity factor. Follow up with the sensor's ID.",
+    required=False,
+    help="To be deprecated. Use consumption-price-sensor instead.",
 )
 @click.option(
     "--start",
@@ -837,8 +854,16 @@ def create_forecasts(
     default=1,
     help="Round-trip efficiency (e.g. 85% or 0.85) to use for the schedule. Defaults to 100% (no losses).",
 )
+@click.option(
+    "--as-job",
+    is_flag=True,
+    help="Whether to queue a scheduling job instead of computing directly. "
+    "To process the job, run a worker (on any computer, but configured to the same databases) to process the 'scheduling' queue. Defaults to False.",
+)
 def create_schedule(
     power_sensor: Sensor,
+    consumption_price_sensor: Sensor,
+    production_price_sensor: Sensor,
     optimization_context_sensor: Sensor,
     start: datetime,
     duration: timedelta,
@@ -847,6 +872,7 @@ def create_schedule(
     soc_min: Optional[ur.Quantity] = None,
     soc_max: Optional[ur.Quantity] = None,
     roundtrip_efficiency: Optional[ur.Quantity] = None,
+    as_job: bool = False,
 ):
     """Create a new schedule for a given power sensor.
 
@@ -856,10 +882,20 @@ def create_schedule(
     - only supports datetimes on the hour or a multiple of the sensor resolution thereafter
     """
 
+    # todo: deprecate the 'optimization-context-id' argument in favor of 'consumption-price-sensor' (announced v0.11.0)
+    tb_utils.replace_deprecated_argument(
+        "optimization-context-id",
+        optimization_context_sensor,
+        "consumption-price-sensor",
+        consumption_price_sensor,
+    )
+
     # Parse input
     if not power_sensor.measures_power:
         click.echo(f"Sensor with ID {power_sensor.id} is not a power sensor.")
         raise click.Abort()
+    if production_price_sensor is None:
+        production_price_sensor = consumption_price_sensor
     end = start + duration
     for attribute in ("min_soc_in_mwh", "max_soc_in_mwh"):
         try:
@@ -901,21 +937,40 @@ def create_schedule(
     if soc_max is not None:
         soc_max = convert_units(soc_max.magnitude, str(soc_max.units), "MWh", capacity=capacity_str)  # type: ignore
 
-    success = make_schedule(
-        sensor_id=power_sensor.id,
-        start=start,
-        end=end,
-        belief_time=server_now(),
-        resolution=power_sensor.event_resolution,
-        soc_at_start=soc_at_start,
-        soc_targets=soc_targets,
-        soc_min=soc_min,
-        soc_max=soc_max,
-        roundtrip_efficiency=roundtrip_efficiency,
-        price_sensor=optimization_context_sensor,
-    )
-    if success:
-        print("New schedule is stored.")
+    if as_job:
+        job = create_scheduling_job(
+            sensor_id=power_sensor.id,
+            start_of_schedule=start,
+            end_of_schedule=end,
+            belief_time=server_now(),
+            resolution=power_sensor.event_resolution,
+            soc_at_start=soc_at_start,
+            soc_targets=soc_targets,
+            soc_min=soc_min,
+            soc_max=soc_max,
+            roundtrip_efficiency=roundtrip_efficiency,
+            consumption_price_sensor=consumption_price_sensor,
+            production_price_sensor=production_price_sensor,
+        )
+        if job:
+            print(f"New scheduling job {job.id} has been added to the queue.")
+    else:
+        success = make_schedule(
+            sensor_id=power_sensor.id,
+            start=start,
+            end=end,
+            belief_time=server_now(),
+            resolution=power_sensor.event_resolution,
+            soc_at_start=soc_at_start,
+            soc_targets=soc_targets,
+            soc_min=soc_min,
+            soc_max=soc_max,
+            roundtrip_efficiency=roundtrip_efficiency,
+            consumption_price_sensor=consumption_price_sensor,
+            production_price_sensor=production_price_sensor,
+        )
+        if success:
+            print("New schedule is stored.")
 
 
 @fm_add_data.command("toy-account")
@@ -970,7 +1025,7 @@ def add_toy_account(kind: str, name: str):
                 )
                 # add charging sensor to battery
                 charging_sensor = Sensor(
-                    name="charging",
+                    name="discharging",
                     generic_asset=asset,
                     unit="MW",
                     timezone="Europe/Amsterdam",
