@@ -201,3 +201,17 @@ def get_or_create_annotation(
     if annotation in db.session:
         db.session.expunge(annotation)
     return existing_annotation
+
+
+def to_annotation_frame(annotations: List[Annotation]) -> pd.DataFrame:
+    """Transform a list of annotations into a DataFrame.
+
+    We don't use a BeliefsDataFrame here, because they are designed for quantitative data only.
+    """
+    return pd.DataFrame(
+        [
+            [a.start, a.end, a.belief_time, a.source, a.type, a.content]
+            for a in annotations
+        ],
+        columns=["start", "end", "belief_time", "source", "type", "content"],
+    )
