@@ -86,3 +86,14 @@ class DataSource(db.Model, tb.BeliefSourceDBMixin):
 
     def __str__(self):
         return self.description
+
+    def to_dict(self) -> dict:
+        model_incl_version = self.model if self.model else ""
+        if self.model and self.version:
+            model_incl_version += f" ({self.version}"
+        return dict(
+            id=self.id,
+            name=self.name,
+            model=model_incl_version,
+            description=self.description,
+        )
