@@ -1,5 +1,6 @@
-from typing import Union, Optional, Tuple
+from typing import Union, Optional, List, Tuple
 import copy
+import json
 
 from flask import url_for, current_app
 from flask_classful import FlaskView
@@ -7,10 +8,11 @@ from flask_wtf import FlaskForm
 from flask_security import login_required, current_user
 from wtforms import StringField, DecimalField, SelectField
 from wtforms.validators import DataRequired
-from flexmeasures.auth.policy import ADMIN_ROLE
+from flexmeasures.auth.policy import user_has_admin_access
 
 from flexmeasures.data import db
 from flexmeasures.auth.error_handling import unauthorized_handler
+from flexmeasures.auth.policy import check_access
 from flexmeasures.data.models.generic_assets import (
     GenericAssetType,
     GenericAsset,
@@ -18,10 +20,8 @@ from flexmeasures.data.models.generic_assets import (
 )
 from flexmeasures.data.models.user import Account
 from flexmeasures.data.models.time_series import Sensor
-from flexmeasures.ui.charts.latest_state import get_latest_power_as_plot
 from flexmeasures.ui.utils.view_utils import render_flexmeasures_template
 from flexmeasures.ui.crud.api_wrapper import InternalApi
-from flexmeasures.utils.unit_utils import is_power_unit
 
 
 """
@@ -39,14 +39,15 @@ class AssetForm(FlaskForm):
     name = StringField("Name")
     latitude = DecimalField(
         "Latitude",
-        places=4,
+        places=None,
         render_kw={"placeholder": "--Click the map or enter a latitude--"},
     )
     longitude = DecimalField(
         "Longitude",
-        places=4,
+        places=None,
         render_kw={"placeholder": "--Click the map or enter a longitude--"},
     )
+    attributes = StringField("Other attributes (JSON)", default="{}")
 
     def validate_on_submit(self):
         if (
@@ -112,6 +113,8 @@ def process_internal_api_response(
     """
     Turn data from the internal API into something we can use to further populate the UI.
     Either as an asset object or a dict for form filling.
+
+    If we add other data by querying the database, we make sure the asset is not in the session afterwards.
     """
 
     def expunge_asset():
@@ -123,20 +126,43 @@ def process_internal_api_response(
     if asset_id:
         asset_data["id"] = asset_id
     if make_obj:
-        asset = GenericAsset(**asset_data)  # TODO: use schema?
+        asset = GenericAsset(
+            **{
+                **asset_data,
+                **{"attributes": json.loads(asset_data.get("attributes", "{}"))},
+            }
+        )  # TODO: use schema?
         asset.generic_asset_type = GenericAssetType.query.get(
             asset.generic_asset_type_id
         )
+        expunge_asset()
+        asset.owner = Account.query.get(asset_data["account_id"])
+        expunge_asset()
         if "id" in asset_data:
-            expunge_asset()
             asset.sensors = Sensor.query.filter(
                 Sensor.generic_asset_id == asset_data["id"]
             ).all()
-        expunge_asset()
+            expunge_asset()
         return asset
     return asset_data
 
 
+def user_can_create_assets() -> bool:
+    try:
+        check_access(current_user.account, "create-children")
+    except Exception:
+        return False
+    return True
+
+
+def user_can_delete(asset) -> bool:
+    try:
+        check_access(asset, "delete")
+    except Exception:
+        return False
+    return True
+
+
 class AssetCrudUI(FlaskView):
     """
     These views help us offer a Jinja2-based UI.
@@ -149,16 +175,36 @@ class AssetCrudUI(FlaskView):
 
     @login_required
     def index(self, msg=""):
-        """/assets"""
-        get_assets_response = InternalApi().get(
-            url_for("AssetAPI:index"), query={"account_id": current_user.account_id}
-        )
-        assets = [
-            process_internal_api_response(ad, make_obj=True)
-            for ad in get_assets_response.json()
-        ]
+        """GET from /assets
+
+        List the user's assets. For admins, list across all accounts.
+        """
+        assets = []
+
+        def get_asset_by_account(account_id) -> List[GenericAsset]:
+            if account_id is not None:
+                get_assets_response = InternalApi().get(
+                    url_for("AssetAPI:index"), query={"account_id": account_id}
+                )
+            else:
+                get_assets_response = InternalApi().get(url_for("AssetAPI:public"))
+            return [
+                process_internal_api_response(ad, make_obj=True)
+                for ad in get_assets_response.json()
+            ]
+
+        if user_has_admin_access(current_user, "read"):
+            for account in Account.query.all():
+                assets += get_asset_by_account(account.id)
+            assets += get_asset_by_account(account_id=None)
+        else:
+            assets = get_asset_by_account(current_user.account_id)
+
         return render_flexmeasures_template(
-            "crud/assets.html", account=current_user.account, assets=assets, message=msg
+            "crud/assets.html",
+            assets=assets,
+            message=msg,
+            user_can_create_assets=user_can_create_assets(),
         )
 
     @login_required
@@ -183,6 +229,7 @@ class AssetCrudUI(FlaskView):
             account=Account.query.get(account_id),
             assets=assets,
             msg=msg,
+            user_can_create_assets=user_can_create_assets(),
         )
 
     @login_required
@@ -190,7 +237,7 @@ class AssetCrudUI(FlaskView):
         """GET from /assets/<id> where id can be 'new' (and thus the form for asset creation is shown)"""
 
         if id == "new":
-            if not current_user.has_role("admin"):
+            if not user_can_create_assets():
                 return unauthorized_handler(None, [])
 
             asset_form = with_options(NewAssetForm())
@@ -210,15 +257,14 @@ class AssetCrudUI(FlaskView):
         asset = process_internal_api_response(asset_dict, int(id), make_obj=True)
         asset_form.process(data=process_internal_api_response(asset_dict))
 
-        latest_measurement_time_str, asset_plot_html = _get_latest_power_plot(asset)
         return render_flexmeasures_template(
             "crud/asset.html",
             asset=asset,
             asset_form=asset_form,
             msg="",
-            latest_measurement_time_str=latest_measurement_time_str,
-            asset_plot_html=asset_plot_html,
             mapboxAccessToken=current_app.config.get("MAPBOX_ACCESS_TOKEN", ""),
+            user_can_create_assets=user_can_create_assets(),
+            user_can_delete_asset=user_can_delete(asset),
         )
 
     @login_required
@@ -264,11 +310,14 @@ class AssetCrudUI(FlaskView):
                         f"Internal asset API call unsuccessful [{post_asset_response.status_code}]: {post_asset_response.text}"
                     )
                     asset_form.process_api_validation_errors(post_asset_response.json())
-                    if (
-                        "message" in post_asset_response.json()
-                        and "json" in post_asset_response.json()["message"]
-                    ):
-                        error_msg = str(post_asset_response.json()["message"]["json"])
+                    if "message" in post_asset_response.json():
+                        asset_form.process_api_validation_errors(
+                            post_asset_response.json()["message"]
+                        )
+                        if "json" in post_asset_response.json()["message"]:
+                            error_msg = str(
+                                post_asset_response.json()["message"]["json"]
+                            )
             if asset is None:
                 msg = "Cannot create asset. " + error_msg
                 return render_flexmeasures_template(
@@ -283,9 +332,6 @@ class AssetCrudUI(FlaskView):
             asset_form = with_options(AssetForm())
             if not asset_form.validate_on_submit():
                 asset = GenericAsset.query.get(id)
-                latest_measurement_time_str, asset_plot_html = _get_latest_power_plot(
-                    asset
-                )
                 # Display the form data, but set some extra data which the page wants to show.
                 asset_info = asset_form.to_json()
                 asset_info["id"] = id
@@ -298,9 +344,9 @@ class AssetCrudUI(FlaskView):
                     asset_form=asset_form,
                     asset=asset,
                     msg="Cannot edit asset.",
-                    latest_measurement_time_str=latest_measurement_time_str,
-                    asset_plot_html=asset_plot_html,
                     mapboxAccessToken=current_app.config.get("MAPBOX_ACCESS_TOKEN", ""),
+                    user_can_create_assets=user_can_create_assets(),
+                    user_can_delete_asset=user_can_delete(asset),
                 )
             patch_asset_response = InternalApi().patch(
                 url_for("AssetAPI:patch", id=id),
@@ -318,18 +364,19 @@ class AssetCrudUI(FlaskView):
                     f"Internal asset API call unsuccessful [{patch_asset_response.status_code}]: {patch_asset_response.text}"
                 )
                 msg = "Cannot edit asset."
-                asset_form.process_api_validation_errors(patch_asset_response.json())
+                asset_form.process_api_validation_errors(
+                    patch_asset_response.json().get("message")
+                )
                 asset = GenericAsset.query.get(id)
 
-        latest_measurement_time_str, asset_plot_html = _get_latest_power_plot(asset)
         return render_flexmeasures_template(
             "crud/asset.html",
             asset=asset,
             asset_form=asset_form,
             msg=msg,
-            latest_measurement_time_str=latest_measurement_time_str,
-            asset_plot_html=asset_plot_html,
             mapboxAccessToken=current_app.config.get("MAPBOX_ACCESS_TOKEN", ""),
+            user_can_create_assets=user_can_create_assets(),
+            user_can_delete_asset=user_can_delete(asset),
         )
 
     @login_required
@@ -347,7 +394,7 @@ def _set_account(asset_form: NewAssetForm) -> Tuple[Optional[Account], Optional[
     account_error = None
 
     if asset_form.account_id.data == -1:
-        if current_user.has_role(ADMIN_ROLE):
+        if user_has_admin_access(current_user, "update"):
             return None, None  # Account can be None (public asset)
         else:
             account_error = "Please pick an existing account."
@@ -381,19 +428,3 @@ def _set_asset_type(
     else:
         current_app.logger.error(asset_type_error)
     return asset_type, asset_type_error
-
-
-def _get_latest_power_plot(asset: GenericAsset) -> Tuple[str, str]:
-    power_sensor: Optional[Sensor] = None
-    if asset._sa_instance_state.transient:
-        sensors = Sensor.query.filter(Sensor.generic_asset_id == asset.id).all()
-    else:
-        sensors = asset.sensors
-    for sensor in sensors:
-        if is_power_unit(sensor.unit):
-            power_sensor = sensor
-            break
-    if power_sensor is None:
-        return "", ""
-    else:
-        return get_latest_power_as_plot(power_sensor)
