@@ -122,9 +122,9 @@ class Config(object):
     FLEXMEASURES_REDIS_DB_NR: int = 0  # Redis per default has 16 databases, [0-15]
     FLEXMEASURES_REDIS_PASSWORD: Optional[str] = None
     FLEXMEASURES_JS_VERSIONS: dict = dict(
-        vega="5",
-        vegaembed="6.17.0",
-        vegalite="5.0.0",
+        vega="5.22.1",
+        vegaembed="6.20.8",
+        vegalite="5.2.0",
         # todo: expand with other js versions used in FlexMeasures
     )
 
